==> apache-plc4x-0.10.0-source-release.zip: apache-plc4x-0.10.0/plc4j/transports/serial/src/main/java/org/apache/plc4x/java/transport/serial/SerialChannelHandler.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.plc4x.java.transport.serial;

import com.fazecast.jSerialComm.SerialPort;
import com.fazecast.jSerialComm.SerialPortDataListener;
import com.fazecast.jSerialComm.SerialPortEvent;
import io.netty.buffer.ByteBuf;

import java.net.SocketAddress;
import java.nio.channels.SelectionKey;

/**
 * This is a wrapper mostly for testing {@link SerialChannel}, @{@link SerialPollingSelector},
 * {@link SerialSelectionKey}, @{@link SerialSelectorProvider} and @{@link SerialSocketChannel}.
 */
public abstract class SerialChannelHandler {

    private final SocketAddress address;

    public SerialChannelHandler(SocketAddress address) {
        this.address = address;
    }

    abstract boolean open();

    abstract String getIdentifier();

    /**
     * This method registers the Callback to the SelectionKey / {@link java.nio.channels.Selector}
     * which is necessary to notify the {@link java.nio.channels.Selector} about
     * available data.
     */
    abstract void registerSelectionKey(SerialSelectionKey selectionKey);

    public abstract void close();

    /**
     *
     * @param buf
     * @return Return the amoubnt of bytes written into the buffer
     */
    public abstract int read(ByteBuf buf);

    /**
     *
     * @return Number of bytes written to wire
     */
    public abstract int write(ByteBuf buf);


    public static class SerialPortHandler extends SerialChannelHandler {

        private SerialChannelConfig config;
        private SerialPort comPort;

        public SerialPortHandler(SocketAddress address, SerialChannelConfig config) {
            super(address);
            this.config = config;
            // Get the serial port described by the path/name in the address.
            comPort = SerialPort.getCommPort(((SerialSocketAddress) address).getIdentifier());
        }

        @Override
        public boolean open() {
            if (comPort.openPort()) {
                comPort.setComPortParameters(config.getBaudRate(), config.getDataBits(),
                    config.getStopBits(), config.getParityBits());
                return true;
            }
            return false;
        }

        @Override
        public String getIdentifier() {
            return comPort.getDescriptivePortName();
        }

        @Override
        public void registerSelectionKey(SerialSelectionKey selectionKey) {
            comPort.addDataListener(new SerialPortDataListener() {
                @Override
                public int getListeningEvents() {
                    return SerialPort.LISTENING_EVENT_DATA_AVAILABLE;
                }

                @Override
                public void serialEvent(SerialPortEvent event) {
                    ((SerialPollingSelector) selectionKey.selector())
                        .addEvent(new SerialPollingSelector.SelectorEvent(selectionKey, SelectionKey.OP_READ));
                }
            });
        }

        @Override
        public void close() {
            this.comPort.closePort();
        }

        @Override
        public int read(ByteBuf buf) {
            int bytesToRead = comPort.bytesAvailable();
            assert bytesToRead > 0;
            byte[] buffer = new byte[bytesToRead];
            comPort.readBytes(buffer, bytesToRead);
            buf.writeBytes(buffer);
            return bytesToRead;
        }

        @Override
        public int write(ByteBuf buf) {
            int expectedToWrite = buf.readableBytes();
            byte[] bytes = new byte[expectedToWrite];
            buf.readBytes(bytes);
            int bytesWritten = comPort.writeBytes(bytes, expectedToWrite);
            return bytesWritten;
        }
    }

}

==> apache-plc4x-0.13.1-source-release.zip: apache-plc4x-0.13.1/plc4j/transports/serial/src/main/java/org/apache/plc4x/java/transport/serial/SerialChannelHandler.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.plc4x.java.transport.serial;

import com.fazecast.jSerialComm.SerialPort;
import com.fazecast.jSerialComm.SerialPortDataListener;
import com.fazecast.jSerialComm.SerialPortEvent;
import io.netty.buffer.ByteBuf;

import java.net.SocketAddress;
import java.nio.channels.SelectionKey;

/**
 * This is a wrapper mostly for testing {@link SerialChannel}, @{@link SerialPollingSelector},
 * {@link SerialSelectionKey}, @{@link SerialSelectorProvider} and @{@link SerialSocketChannel}.
 */
public abstract class SerialChannelHandler {

    private final SocketAddress address;

    public SerialChannelHandler(SocketAddress address) {
        this.address = address;
    }

    abstract boolean open();

    abstract String getIdentifier();

    /**
     * This method registers the Callback to the SelectionKey / {@link java.nio.channels.Selector}
     * which is necessary to notify the {@link java.nio.channels.Selector} about
     * available data.
     */
    abstract void registerSelectionKey(SerialSelectionKey selectionKey);

    public abstract void close();

    /**
     *
     * @param buf
     * @return Return the amoubnt of bytes written into the buffer
     */
    public abstract int read(ByteBuf buf);

    /**
     *
     * @return Number of bytes written to wire
     */
    public abstract int write(ByteBuf buf);


    public static class SerialPortHandler extends SerialChannelHandler {

        private final SerialChannelConfig config;
        private final SerialPort comPort;

        public SerialPortHandler(SocketAddress address, SerialChannelConfig config) {
            super(address);
            this.config = config;
            // Get the serial port described by the path/name in the address.
            comPort = SerialPort.getCommPort(((SerialSocketAddress) address).getIdentifier());
        }

        @Override
        public boolean open() {
            if (comPort.openPort()) {
                comPort.setComPortParameters(config.getBaudRate(), config.getDataBits(),
                    config.getStopBits(), config.getParity().value);
                return true;
            }
            return false;
        }

        @Override
        public String getIdentifier() {
            return comPort.getDescriptivePortName();
        }

        @Override
        public void registerSelectionKey(SerialSelectionKey selectionKey) {
            comPort.addDataListener(new SerialPortDataListener() {
                @Override
                public int getListeningEvents() {
                    return SerialPort.LISTENING_EVENT_DATA_AVAILABLE;
                }

                @Override
                public void serialEvent(SerialPortEvent event) {
                    ((SerialPollingSelector) selectionKey.selector())
                        .addEvent(new SerialPollingSelector.SelectorEvent(selectionKey, SelectionKey.OP_READ));
                }
            });
        }

        @Override
        public void close() {
            this.comPort.closePort();
        }

        @Override
        public int read(ByteBuf buf) {
            int bytesToRead = comPort.bytesAvailable();
            assert bytesToRead > 0;
            byte[] buffer = new byte[bytesToRead];
            comPort.readBytes(buffer, bytesToRead);
            buf.writeBytes(buffer);
            return bytesToRead;
        }

        @Override
        public int write(ByteBuf buf) {
            int expectedToWrite = buf.readableBytes();
            byte[] bytes = new byte[expectedToWrite];
            buf.readBytes(bytes);
            int bytesWritten = comPort.writeBytes(bytes, expectedToWrite);
            return bytesWritten;
        }
    }

}
